"""
Modern Algorithms (2022–2025)
─────────────────────────────
AVOA — African Vultures Optimization Algorithm (2022)
DMO  — Dwarf Mongoose Optimization (2022)
MGO  — Mountain Gazelle Optimizer (2022)
DBO  — Dung Beetle Optimizer (2023)
COA  — Coati Optimization Algorithm (2023)
OOA  — Osprey Optimization Algorithm (2023)
NOA  — Nutcracker Optimization Algorithm (2023)
SAO  — Snow Ablation Optimizer (2023)
FLA  — Fick's Law Algorithm (2023)
EVO  — Energy Valley Optimizer (2023)
EDO  — Exponential Distribution Optimizer (2023)
MOA  — Mother Optimization Algorithm (2023)
CPO  — Crested Porcupine Optimizer (2024)
PO   — Parrot Optimizer (2024)
FO   — Fox Optimizer (2024)
HO   — Hippopotamus Optimization Algorithm (2024)
KOA  — Kepler Optimization Algorithm (2024)
SBOA — Secretary Bird Optimization Algorithm (2024)
GMO  — Geometric Mean Optimizer (2024)
FFO  — Fennec Fox Optimization (2024)
"""

from heurilab.algorithms.modern.avoa import AVOA
from heurilab.algorithms.modern.dmo import DMO
from heurilab.algorithms.modern.mgo import MGO
from heurilab.algorithms.modern.dbo import DBO
from heurilab.algorithms.modern.coa import COA
from heurilab.algorithms.modern.ooa import OOA
from heurilab.algorithms.modern.noa import NOA
from heurilab.algorithms.modern.sao import SAO
from heurilab.algorithms.modern.fla import FLA
from heurilab.algorithms.modern.evo import EVO
from heurilab.algorithms.modern.edo import EDO
from heurilab.algorithms.modern.moa import MOA
from heurilab.algorithms.modern.cpo import CPO
from heurilab.algorithms.modern.po import PO
from heurilab.algorithms.modern.fo import FO
from heurilab.algorithms.modern.ho import HO
from heurilab.algorithms.modern.koa import KOA
from heurilab.algorithms.modern.sboa import SBOA
from heurilab.algorithms.modern.gmo import GMO
from heurilab.algorithms.modern.ffo import FFO

__all__ = [
    "AVOA", "DMO", "MGO",
    "DBO", "COA", "OOA", "NOA", "SAO", "FLA", "EVO", "EDO", "MOA",
    "CPO", "PO", "FO", "HO", "KOA", "SBOA", "GMO", "FFO",
]
